# Generated by Django 3.0.5 on 2020-09-08 14:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions_api', '0003_subscriptionplan_feature_ref'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscriptionplan',
            name='features',
            field=models.TextField(blank=True, help_text='list of allowed features for this plan', null=True),
        ),
        migrations.AddField(
            model_name='subscriptionplan',
            name='trial_period',
            field=models.PositiveIntegerField(default=0, help_text='how many days to give free before the subscription subscription begins'),
        ),
    ]
