# Generated by Django 3.0.5 on 2020-05-12 14:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions_api', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='planlistdetail',
            options={'ordering': ('order',)},
        ),
        migrations.AddField(
            model_name='planlist',
            name='features_ref',
            field=models.CharField(blank=True, help_text='Reference to select list of features to display for the plan list', max_length=12, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='planlistdetail',
            name='html_content',
            field=models.TextField(blank=True, help_text='HTML content can also be json object to display for plan', null=True),
        ),
    ]
